@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# $Id: wlink.py 1229 2015-01-27 20:16:18Z mwall $
+# $Id: wlink.py 1395 2016-01-19 15:43:12Z mwall $
 # Copyright 2014 Matthew Wall
 """weewx driver for WeatherLink web sites
    download data from a weatherlink site for use in weewx
@@ -49,7 +49,8 @@ from HTMLParser import HTMLParser
 import weewx
 import weewx.drivers
 
-DRIVER_VERSION = "0.4"
+DRIVER_NAME = "WeatherLink"
+DRIVER_VERSION = "0.7"
 
 if weewx.__version__ < "3":
     raise weewx.UnsupportedFeature("weewx 3 is required, found %s" %
@@ -70,6 +71,10 @@ def logcrt(msg):
 def logerr(msg):
     logmsg(syslog.LOG_ERR, msg)
 
+def logdev(msg):
+#    print msg
+    pass
+
 def loader(config_dict, engine):
     return WeatherLink(**config_dict['WeatherLink'])
 
@@ -78,6 +83,10 @@ class WeatherLink(weewx.drivers.AbstractDevice):
     
     Parse the web page for 'loop' data.  This happens every 60 seconds.
 
+    There are two formats:
+    A - pre-2015; uses div and span within td
+    B - end of 2015; elimited divs and inline styles
+
     For archive data, make a request to the server.
     """
 
@@ -85,9 +94,11 @@ class WeatherLink(weewx.drivers.AbstractDevice):
         loginf("version is %s" % DRIVER_VERSION)
         self.username = stn_dict['username']
         self.password = stn_dict['password']
+        self.format = float(stn_dict.get('format', 'B'))
         self.poll_interval = float(stn_dict.get('poll_interval', 60))
         self.max_tries = int(stn_dict.get('max_tries', 5))
         self.retry_wait = int(stn_dict.get('retry_wait', 30))
+        self.raw_data_cache = stn_dict.get('raw_data_cache', None)
         self._archive_interval = stn_dict.get('archive_interval', None)
         if self._archive_interval is not None:
             self._archive_interval = int(self._archive_interval)
@@ -113,6 +124,9 @@ class WeatherLink(weewx.drivers.AbstractDevice):
                 'dateTime': int(time.time() + 0.5),
                 'usUnits': weewx.US, # type 2 is US imperial
                 }
+            if self.raw_data_cache is not None:
+                with open(self.raw_data_cache, "w") as f:
+                    f.write(data)
             packet.update(self.parse_page(data))
             yield packet
             time.sleep(self.poll_interval)
@@ -154,7 +168,7 @@ class WeatherLink(weewx.drivers.AbstractDevice):
         """parse weather data from a weatherlink web page"""
         packet = {}
         if text is not None:
-            parser = WLParser()
+            parser = WLParserB()
             parser.feed(text)
             packet = parser.get_data()
         return packet
@@ -365,13 +379,14 @@ rec_fmt_B = struct.Struct('<' + ''.join(fmt_B))
 compass_points = {'N':0,'NNE':22.5,'NE':45,'ENE':67.5,'E':90,'ESE':112.5,'SE':135,'SSE':157.5,'S':180,'SSW':202.5,'SW':225,'WSW':247.5,'W':270,'WNW':292.5,'NW':315,'NNW':337.5}
 
 # crude parser to get data from the weatherlink web page.
-class WLParser(HTMLParser):
+# works with weatherlink html as of 2016 (pages changed sometime late 2015)
+# pick off temperature, wind, humidity, rain, barometer
+class WLParserB(HTMLParser):
     def __init__(self):
         HTMLParser.__init__(self)
         self.packet = {}
         self.hierarchy = []
-        self.last_value = None
-        self.last_key = None
+        self.last_key = 'Temperature'
 
     def handle_starttag(self, tag, attrs):
         if tag not in ['meta','link','br']:
@@ -381,43 +396,70 @@ class WLParser(HTMLParser):
         self.hierarchy.pop()
 
     def handle_data(self, data):
-        data = data.strip()
-        if not 'span' in self.hierarchy:
-            if len(data):
-                self.last_key = data
+        if not 'body' in self.hierarchy:
             return
-        if data in compass_points:
-            self.packet['windDir'] = compass_points[data]
-        elif self.last_value is not None:
-            if data == self.last_value:
-                try:
-                    v = float(self.last_value)
-                    self.packet['outTemperature'] = v
-                except:
-                    pass
-            elif data == 'km/h' or data == 'Mph':
-                self.packet['windSpeed'] = float(self.last_value)
-            elif data == '%':
-                self.packet['outHumidity'] = float(self.last_value)
-            self.last_value = None
-        elif re.search('\d+mm', data):
-            m = re.search('(\d+)mm', data)
-            self.packet['rain'] = float(m.group(1)) / 10 # weewx wants cm
-        elif re.search('[\d.]+mb', data):
-            m = re.search('([\d.]+)mb', data)
-            self.packet['barometer'] = float(m.group(1))
-        elif re.search('[\d.]+hPa', data):
-            m = re.search('([\d.]+)hPa', data)
-            self.packet['barometer'] = float(m.group(1))
-        elif re.search('[\d.]+"', data):
-            m = re.search('([\d.]+)"', data)
-            if self.last_key == 'Rain':
+        logdev('%s' % self.hierarchy)
+        data = data.strip()
+        logdev('%s' % data)
+        if len(data) == 0:
+            return
+        if data in ['Wind', 'Humidity', 'Rain', 'Barometer']:
+            self.last_key = data
+            logdev('found key %s' % self.last_key)
+            return
+
+        if self.last_key == 'Wind':
+            if data == 'Calm':
+                self.packet['windSpeed'] = 0
+                self.packet['windDir'] = None
+            elif re.search('[NSEW]+', data):
+                m = re.search('([NSEW]+)', data)
+                wdir = m.group(1)
+                if wdir in compass_points:
+                    self.packet['windDir'] = compass_points[wdir]
+                logdev('windDir: %s' % self.packet.get('windDir'))
+                # do not reset last_key since we still need a wind speed
+            elif re.search('\d+', data):
+                m = re.search('(\d+)', data)
+                self.packet['windSpeed'] = m.group(1)
+                logdev('windSpeed: %s' % self.packet.get('windSpeed')) 
+                self.last_key = None
+        if self.last_key == 'Temperature':
+            try:
+                v = float(data)
+                self.packet['outTemperature'] = v
+                logdev('outTemperature: %s' % self.packet['outTemperature'])
+                self.last_key = None
+            except:
+                pass
+        if self.last_key == 'Humidity':
+            self.packet['outHumidity'] = float(data)
+            logdev('outHumidity: %s' % self.packet['outHumidity'])
+            self.last_key = None
+        if self.last_key == 'Rain':
+            if re.search('\d+mm', data):
+                m = re.search('(\d+)mm', data)
+                self.packet['rain'] = float(m.group(1)) / 10 # weewx wants cm
+                logdev('rain: %s' % self.packet['rain'])
+            elif re.search('[\d.]+"', data):
+                m = re.search('([\d.]+)"', data)
                 self.packet['rain'] = float(m.group(1))
-            elif self.last_key == 'Barometer':
+                logdev('rain: %s' % self.packet['rain'])
+            self.last_key = None
+        if self.last_key == 'Barometer':
+            if re.search('[\d.]+mb', data):
+                m = re.search('([\d.]+)mb', data)
                 self.packet['barometer'] = float(m.group(1))
+                logdev('barometer: %s' % self.packet['barometer']) # FIXME
+            elif re.search('[\d.]+hPa', data):
+                m = re.search('([\d.]+)hPa', data)
+                self.packet['barometer'] = float(m.group(1))
+                logdev('barometer: %s' % self.packet['barometer']) # FIXME
+            elif re.search('[\d.]+"', data):
+                m = re.search('([\d.]+)"', data)
+                self.packet['barometer'] = float(m.group(1))
+                logdev('barometer: %s' % self.packet['barometer'])
             self.last_key = None
-        else:
-            self.last_value = data
 
     def get_data(self):
         return self.packet
@@ -428,27 +470,29 @@ if __name__ == "__main__":
     usage = """%prog [options]"""
     import optparse
     parser = optparse.OptionParser(usage=usage)
-    parser.add_option('--username', dest='username',
+    parser.add_option('--username', dest='username', default='user',
                       help='weatherlink username')
-    parser.add_option('--password', dest='password',
+    parser.add_option('--password', dest='password', default='pass',
                       help='weatherlink password')
     parser.add_option('--test-driver', dest='test_driver', action='store_true',
                       help='test the driver')
     parser.add_option('--test-parser', dest='test_parser', action='store_true',
                       help='test the parser')
+    parser.add_option('--filename', dest='filename', default='testfile.xml',
+                      help='name of file for test-parser')
     (options, args) = parser.parse_args()
+    syslog.setlogmask(syslog.LOG_UPTO(syslog.LOG_DEBUG))
     if options.test_parser:
         data = []
-        with open('testfile.xml') as f:
+        with open(options.filename) as f:
             for line in f:
                 data.append(line)
-        parser = WLParser()
+        parser = WLParserB()
         parser.feed(''.join(data))
         print parser.get_data()
-    else:
+    elif options.test_driver:
         import weeutil.weeutil
         station = WeatherLink(username=options.username,
                               password=options.password)
         for p in station.genLoopPackets():
             print weeutil.weeutil.timestamp_to_string(p['dateTime']), p
-
